/*
 *  filelog.c - Contains a function logging information to a file
 *
 *  Copyright (C) 1998 Hugo Haas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>

#include "log.h"

extern struct logger log;

/* Mutex */
pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

/*
 * log_entry
 *
 * Log an entry into a stream
 */

void log_entry(int fd, char *entry) {
  time_t current = time(NULL);
  char date[27];

  sprintf(date, asctime(localtime(&current)));
  write(fd, date, strlen(date)-6);
  write(fd, " ", 1);
  write(fd, entry, (strlen(entry) < 1023) ? strlen(entry) : 1023 );
  write(fd, "\n", 1);
}

/*
 * filelog
 *
 * Log information into a file
 */

void filelog(int priority, char *format, ...) {
  int f;

  pthread_mutex_lock(&mutex);

  f = open((char *) log.file, O_WRONLY | O_APPEND | O_CREAT, 0640);

  if (f>0) {
    char buffer[1024];
    va_list msg;

    va_start(msg,format);
    vsprintf(buffer,format,msg);
    va_end(msg);

    log_entry(f, buffer);

    close(f);
  }
  pthread_mutex_unlock(&mutex);
}

/*
 * stdout
 *
 * Displays information onto stdout
 */

void stdoutlog(int priority, char *format, ...) {
   char buffer[1024];
   va_list msg;

   va_start(msg,format);
   vsprintf(buffer,format,msg);
   va_end(msg);

   pthread_mutex_lock(&mutex);
   log_entry(1, buffer);
   pthread_mutex_unlock(&mutex);

   fflush(stdout);
}

/*
 * $Log: log.c,v $
 * Revision 1.1  1998/10/25 15:05:00  hugo
 * Initial revision
 *
 */
